#include "input.h"
#include "gfx.hpp"

LPDIRECTINPUT8 g_pDI;
CZenKeyboard g_Keyboard;
CZenMouse g_Mouse;


HRESULT InitializeInput(LPDIRECT3DDEVICE8 pD3DDevice,HWND hWndMain,HINSTANCE hInstMain)
{
  if( g_pDI )
    g_pDI->Release();

  HRESULT r = 0;

  // Create the IDirectInput8 object
  r = DirectInput8Create( hInstMain, DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&g_pDI, NULL );
  if( FAILED( r ) )
  {
    SetError( "Failed to create DirectInput" );
    return E_FAIL;
  }

  // Initialize the keyboard
  r = g_Keyboard.Initialize(pD3DDevice,g_pDI,hWndMain);
  if( FAILED( r ) )
  {	
    SetError( "Failed to initialize keyboard" );
    return E_FAIL;
  }	

  // Initialize the mouse
  r = g_Mouse.Initialize(pD3DDevice,g_pDI,hWndMain);
  if( FAILED( r ) )
  {
    SetError( "Unable to initialize mouse" );
    return E_FAIL;
  }

  return S_OK;
}

HRESULT ShutdownInput()
{
  // Get rid of the IDirectInput8 object
  if( g_pDI )
  {
    g_pDI->Release();
    g_pDI = 0;
  }

  return E_FAIL;
}

CZenMouse::CZenMouse()
{
	m_pMouseDev = 0;

	m_bInitialized = FALSE;

	m_bShowCursor = FALSE;

	m_pCursorSurf = 0;

	m_Position.x = m_Position.y = 0;
}

CZenMouse::~CZenMouse()
{
	if( m_pCursorSurf )
		m_pCursorSurf->Release();
	
	// Unacquire and release the mouse device
	if( m_pMouseDev )
	{
		m_pMouseDev->Unacquire();
		m_pMouseDev->Release();
	}
}

// Set the position of the cursor, in screen coords
void CZenMouse::SetCursorPosition( int x, int y )
{
	
	// Make sure the point stays within the screen
	if( x < 0 )
		x = 0;

	if( y < 0 )
		y = 0;

	
	if( x > 800-1 )
		x = 800-1;

	if( y > 600-1 )
		y = 600-1;
	

	// Update the postion tracker
	m_Position.x = x;
	m_Position.y = y;

	// Tell Direct3D about the new position
	g_pD3DDevice->SetCursorPosition( x, y, 0 );
}

// Returns the current position of the cursor
void CZenMouse::GetCursorPosition( int& x, int& y )
{
	x = m_Position.x;
	y = m_Position.y;
}

// Moves the cursor relative to its current position
void CZenMouse::MoveCursor( int x, int y )
{
	m_Position.x += x;
	m_Position.y += y;

	g_pD3DDevice->SetCursorPosition( x, y, 0 );
}

// Sets the mouse cursor position
// by tracking how far it has moved
// since the last update.
void CZenMouse::UpdateCursorPos()
{
	// Get the relative movement
	// out of the DIMOUSESTATE structure
	m_Position.x += m_MouseData.lX;
	m_Position.y += m_MouseData.lY;

	// Make sure the point is within screen bounds
	if( m_Position.x < 0 )
		m_Position.x = 0;

	if( m_Position.y < 0 )
		m_Position.y = 0;

	if( m_Position.x > 800-1 )
		m_Position.x = 800-1;

	if( m_Position.y > 600-1 )
		m_Position.y = 600-1;

	// Set the new position into the device
	g_pD3DDevice->SetCursorPosition( m_Position.x, m_Position.y, 0 );	
}

// Handles the WM_SETCURSOR message
BOOL CZenMouse::HandleSetCursor()
{
	// Return if the mouse hasnt been intialized yet
	if( !m_bInitialized )
		return FALSE;
	
	// If the cursor is set to be visible...
	if( m_bShowCursor )
	{
		// Turn off standard cursor 
		SetCursor( NULL );
		// Show the zen cursor
		g_pD3DDevice->ShowCursor( TRUE );

		// Return TRUE, which prevents
		// windows from messing with the
		// cursors anymore
		return TRUE;
	}
			
	// Return FALSE to let windows do
	// its thing if our cursor is not visible
	return FALSE;
}

POINT CZenMouse::GetMousePos()
{
	// Holds mouse data
	POINT MousePos;

	// Get the data from the buffer
	MousePos.x = m_MouseData.lX;
	MousePos.y = m_MouseData.lY;
	
	// Return the position
	return MousePos;
}

BOOL CZenMouse::IsButtonDown( int Button )
{
	// Return the button status from the buffer	
	if( m_MouseData.rgbButtons[Button] & 0x80 )
		return TRUE;
	else
		return FALSE;
}

HRESULT CZenMouse::Poll()
{
	HRESULT r = 0;

	// Return if the object has not been initialized
	if( !m_bInitialized )
		return E_FAIL;

	// Get the state of the mouse
	r = m_pMouseDev->GetDeviceState( sizeof( DIMOUSESTATE ), &m_MouseData );
	if( FAILED( r ) )
	{
		// If the mouse has moved focus
		if( r == DIERR_INPUTLOST )
		{
			// Reacquire the mouse
			while( r == DIERR_INPUTLOST )
				r = m_pMouseDev->Acquire();

			// Try to test the state again
			if( SUCCEEDED( r ) )
				m_pMouseDev->GetDeviceState( sizeof( DIMOUSESTATE ), &m_MouseData );
			else
				return FALSE;
		}
		else
			return E_FAIL;
	}

	return S_OK;
}

// Initializes the mouse
HRESULT CZenMouse::Initialize(LPDIRECT3DDEVICE8 pD3DDevice,LPDIRECTINPUT8 pDI, HWND hWndMain)
{
	HRESULT r = 0;

	// Return if the DirectInput object does not exist
	if( !pDI )
		return E_FAIL;

	// Release the mouse device if it has already been created
	if( m_pMouseDev )
		m_pMouseDev->Release();

	// Create the mouse device
	r = pDI->CreateDevice( GUID_SysMouse, &m_pMouseDev, NULL );
	if( FAILED( r ) )
	{
		SetError( "Unable to create mouse device" );
		return E_FAIL;
	}

	// Set the data format for the mouse
	r = m_pMouseDev->SetDataFormat( &c_dfDIMouse );
	if( FAILED( r ) )
	{
		SetError( "Unable to set the mouse data format" );
		return E_FAIL;
	}

	// Set the cooperative level for the mouse
	r = m_pMouseDev->SetCooperativeLevel( hWndMain, DISCL_EXCLUSIVE | DISCL_FOREGROUND );
	//r = m_pMouseDev->SetCooperativeLevel( hWndMain, DISCL_NONEXCLUSIVE | DISCL_FOREGROUND );
	if( FAILED( r ) )
	{
		SetError( "Unable to set the cooperative level for the mouse" );
		return E_FAIL;
	}

	// Acquire the physical mouse into the device
	r = m_pMouseDev->Acquire();
	if( FAILED( r ) )
	{
		SetError( "Unable to acquire mouse" );
		return E_FAIL;
	}

	// Create a new surface for the mouse pointer image
	pD3DDevice->CreateImageSurface( 32, 64, D3DFMT_A8R8G8B8, &m_pCursorSurf );
	// Load the image file from disk
	r=D3DXLoadSurfaceFromFile( m_pCursorSurf, 0, 0, "media\\cursor.jpg", 0, D3DX_FILTER_NONE, 0, 0 );
	// Set the hotspot for the cursor
	r=pD3DDevice->SetCursorProperties( 0, 0, m_pCursorSurf );
	
	// Set the initialization flag to true
	m_bInitialized = TRUE;

	return S_OK;
}

CZenKeyboard::CZenKeyboard()
{
	ZeroMemory( &m_KeyBuffer, sizeof( m_KeyBuffer ) );
	m_pKeyDev = 0;

	m_bInitialized = FALSE;
}

CZenKeyboard::~CZenKeyboard()
{
	if( m_pKeyDev )
	{
		m_pKeyDev->Unacquire();
		
		m_pKeyDev->Release();
	}
}

HRESULT CZenKeyboard::Initialize(LPDIRECT3DDEVICE8 pD3DDevice,LPDIRECTINPUT8 pDI,HWND hWndMain)
{
	HRESULT r;

	if( !pDI )
		return E_FAIL;

	// Release the device if it has already been created
	if( m_pKeyDev )
	{
		m_pKeyDev->Unacquire();
		m_pKeyDev->Release();
	}
	
	// Create the device for the keyboard
	r = pDI->CreateDevice( GUID_SysKeyboard, &m_pKeyDev, NULL );
	if( FAILED( r ) )
	{
		SetError( "Failed to create key device" );
		return E_FAIL;
	}

	// Set the data format for the device
	r = m_pKeyDev->SetDataFormat( &c_dfDIKeyboard );
	if( FAILED( r ) )
	{
		SetError( "Unable to set the keyboard data format" );
		return E_FAIL;
	}

	// Set the cooperative level
	r = m_pKeyDev->SetCooperativeLevel( hWndMain, DISCL_BACKGROUND | DISCL_NONEXCLUSIVE );
	if( FAILED( r ) )
	{
		SetError( "Unable to set keyboard cooperative level" );
		return E_FAIL;
	}
	
	// Acquire the device
	r = m_pKeyDev->Acquire();
	if( FAILED( r ) )
	{
		SetError( "Unable to acquire the keyboard" );
		return E_FAIL;
	}

	// Set the initialization flag to true
	m_bInitialized = TRUE;

	return S_OK;
}

BOOL CZenKeyboard::IsKeyDown( int Key )
{
	HRESULT r = 0;

	// Make sure the keyboard has been initialized
	if( !m_bInitialized )
		return FALSE;

	// Get the state of the keyboard into the key buffer
	r = m_pKeyDev->GetDeviceState( sizeof( m_KeyBuffer ), &m_KeyBuffer );
	if( FAILED( r ) )
	{
		// If the device is not acquired...
		if( r == DIERR_INPUTLOST )
		{
			// ...then reacquire the device
			while( r == DIERR_INPUTLOST )
				r = m_pKeyDev->Acquire();

			if( SUCCEEDED( r ) )
				m_pKeyDev->GetDeviceState( sizeof( m_KeyBuffer ), &m_KeyBuffer );
			else
				return FALSE;
		}
		else
			// ...Otherwise it was some other error
			return FALSE;
	}

	// Check if the key was set
	if( m_KeyBuffer[Key] & 0x80 )
		return TRUE;
	else
		return FALSE;
}

BOOL CZenKeyboard::IsAnyKeyDown()
{HRESULT r = 0;

	// Make sure the keyboard has been initialized
	if( !m_bInitialized )
		return FALSE;

	// Get the state of the keyboard into the key buffer
	r = m_pKeyDev->GetDeviceState( sizeof( m_KeyBuffer ), &m_KeyBuffer );
	if( FAILED( r ) )
	{
		// If the device is not acquired...
		if( r == DIERR_INPUTLOST )
		{
			// ...then reacquire the device
			while( r == DIERR_INPUTLOST )
				r = m_pKeyDev->Acquire();

			if( SUCCEEDED( r ) )
				m_pKeyDev->GetDeviceState( sizeof( m_KeyBuffer ), &m_KeyBuffer );
			else
				return FALSE;
		}
		else
			// ...Otherwise it was some other error
			return FALSE;
	}

  for( int i=0; i<256; i++ )
  {
	  if( m_KeyBuffer[i] & 0x80 )
		  return TRUE;
	
  }

	return FALSE;
}
